require "spec_helper"

describe Money, "core extensions" do

  describe Numeric do
    describe "#to_money" do
      it "work as documented" do
        money = 12.34.to_money
        money.amount.should == 12.34
        money.currency.should == Money.default_currency

        money = 100.37.to_money
        money.amount.should == 100.37
        money.currency.should == Money.default_currency

        money = BigDecimal.new('1234').to_money
        money.amount.should == 1234.00
        money.currency.should == Money.default_currency
      end

      it "accepts optional currency" do
        1234.to_money('USD').should == Money.new(1234.00, 'USD')
        1234.to_money('EUR').should == Money.new(1234.00, 'EUR')
      end

      it "respects :subunit_to_unit currency property" do
        10.to_money('USD').should == Money.new(10.00, 'USD')
        10.to_money('TND').should == Money.new(10.000, 'TND')
        10.to_money('CLP').should == Money.new(10, 'CLP')
      end

      specify "GH-15" do
        amount = 555.55.to_money
        amount.should == Money.new(555.55)
      end
    end
  end

  describe String do
    describe "#to_money" do
      it "accepts optional currency" do
        "10.10".to_money('USD').should == Money.new(10.10, 'USD')
        "10.10".to_money('EUR').should == Money.new(10.10, 'EUR')
        "10.10".to_money('USD').should == Money.new(10.10, 'USD')
      end

      it "respects :subunit_to_unit currency property" do
        "1".to_money("USD").should == Money.new(1.00,  "USD")
        "1".to_money("TND").should == Money.new(1.000, "TND")
        "1".to_money("CLP").should == Money.new(1,     "CLP")
        "1.5".to_money("KWD").amount.should == 1.500
      end

      it "rejects invalid amounts" do
        lambda { "not a 12 number".to_money }.should raise_exception(ArgumentError, /Invalid amount/)
      end
    end

    describe "#to_currency" do
      it "converts String to Currency" do
        "USD".to_currency.should == Money::Currency.new("USD")
        "EUR".to_currency.should == Money::Currency.new("EUR")
      end

      it "raises Money::Currency::UnknownCurrency with unknown Currency" do
        expect { "XXX".to_currency }.to raise_error(Money::Currency::UnknownCurrency)
        expect { " ".to_currency }.to raise_error(Money::Currency::UnknownCurrency)
      end
    end
  end

  describe Symbol do
    describe "#to_currency" do
      it "converts Symbol to Currency" do
        :usd.to_currency.should == Money::Currency.new("USD")
        :ars.to_currency.should == Money::Currency.new("ARS")
      end

      it "is case-insensitive" do
        :EUR.to_currency.should == Money::Currency.new("EUR")
      end

      it "raises Money::Currency::UnknownCurrency with unknown Currency" do
        expect { :XXX.to_currency }.to raise_error(Money::Currency::UnknownCurrency)
        expect { :" ".to_currency }.to raise_error(Money::Currency::UnknownCurrency)
      end
    end
  end

end
