$:.unshift File.expand_path(File.dirname(__FILE__) + '/../lib')

require 'test/unit'
require 'json'
require 'sc_core/supply_chain'
require 'sc_core/message_bus'
require 'mocha'
require 'uuidtools'
require 'sc_mq'
require 'sc_mq/client/mq_client'

SERVICE_NAME = 'test'

class MessageBusTest < Test::Unit::TestCase
  
  include ScCore::CommandHandler
  include ScCore::EventHandler
  
  class TestMessageHandler
    extend ScCore::EventHandler
  end
  
  def setup
    ScCore::MessageBus.handlers.clear
  end
    
  def test_subscribe
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    ScCore::MessageBus.subscribe(:test_message1, TestMessageHandler.new, :test_message1)
    ScCore::MessageBus.subscribe(:test_message2, TestMessageHandler.new, :test_message2)
    assert_equal 3, ScCore::MessageBus.handlers.size
    assert_equal 2, ScCore::MessageBus.handlers[:test_message].size
  end
  
  def test_sync_send
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    TestMessageHandler.any_instance.stubs(:test_message).once
    ScCore::MessageBus.send(:test_message, {:arg1 => :value1})
  end
  
  def test_sync_send_multiple_messages_subscribed
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    ScCore::MessageBus.subscribe(:test_message1, TestMessageHandler.new, :test_message)
    TestMessageHandler.any_instance.stubs(:test_message).once
    TestMessageHandler.any_instance.stubs(:test_message1).never
    ScCore::MessageBus.send(:test_message, {:arg1 => :value1})
  end
  
  def test_sync_send_multiple_handlers_only_first_gets_invoked
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    TestMessageHandler.any_instance.stubs(:test_message).once
    ScCore::MessageBus.send(:test_message, {:arg1 => :value1})
  end
  
  def test_sync_send_no_handlers
    TestMessageHandler.any_instance.stubs(:test_message).never
    ScCore::MessageBus.send(:test_message, {:arg1 => :value1})
  end
  
  def test_sync_publish
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    TestMessageHandler.any_instance.stubs(:test_message).twice
    ScCore::MessageBus.publish(:test_message, {:arg1 => :value1})
  end
  
  def test_sync_publish_multiple_messages_subscribed
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    ScCore::MessageBus.subscribe(:test_message, TestMessageHandler.new, :test_message)
    ScCore::MessageBus.subscribe(:test_message1, TestMessageHandler.new, :test_message)
    TestMessageHandler.any_instance.stubs(:test_message).twice
    ScCore::MessageBus.publish(:test_message, {:arg1 => :value1})
  end
  
  def test_sync_publish_no_handlers
    TestMessageHandler.any_instance.stubs(:test_message).never
    ScCore::MessageBus.publish(:test_message, {:arg1 => :value1})
  end
  
  def test_async_send_missing_input
    assert_raises(ScCore::MessageBusError) {ScCore::MessageBus.send(:test_message, {:request_uri => 'http://flipkart.com', :http_method => 'GET', :payload => {}}, {:async => true})}
    assert_raises(ScCore::MessageBusError) {ScCore::MessageBus.send(:test_message, {:exchange_name => 'test_exchange', :http_method => 'GET', :payload => {}}, {:async => true})}
    assert_raises(ScCore::MessageBusError) {ScCore::MessageBus.send(:test_message, {:request_uri => 'http://flipkart.com', :exchange_name => 'test_exchange', :payload => {}}, {:async => true})}
    assert_raises(ScCore::MessageBusError) {ScCore::MessageBus.send(:test_message, {:request_uri => 'http://flipkart.com', :http_method => 'GET', :exchange_name => 'test_exchange'}, {:async => true})}
  end
  
  def test_async_send
    args = {:request_uri => 'http://flipkart.com', :http_method => 'GET', :payload => {}, :exchange_name => 'test_exchange'}
    ScMq::Client::MqClient.any_instance.stubs(:send_message).with(args[:payload], args[:http_method], args[:request_uri], {}).once
    ScCore::MessageBus.send(:test_message, args, {:async => true})
  end
  
  def test_async_publish_missing_input
    assert_raises(ScCore::MessageBusError) {ScCore::MessageBus.publish(:test_message, {:tag => 'test.tag'}, {:async => true})}
    assert_raises(ScCore::MessageBusError) {ScCore::MessageBus.publish(:test_message, {:payload => {}}, {:async => true})}
  end
  
  def test_async_publish
    args = {:tag => 'test.tag', :payload => {}}
    ScMq::Client::MqClient.any_instance.stubs(:send_message).with(args[:payload].to_json, nil, nil, {}, nil, nil, args[:tag]).once
    ScCore::MessageBus.publish(:test_message, args, {:async => true})
  end
  
  def test_command_handler_handles
    handles :test_command
    assert ScCore::MessageBus.handlers.size == 1
    assert ScCore::MessageBus.handlers[:test_command].size == 1
    assert_equal :test_command, ScCore::MessageBus.handlers[:test_command].first[:method]
  end
  
  def test_command_handler_handles_with_opts
    handles :test_command, :with => :test_command1
    assert ScCore::MessageBus.handlers.size == 1
    assert ScCore::MessageBus.handlers[:test_command].size == 1
    assert_equal :test_command1, ScCore::MessageBus.handlers[:test_command].first[:method]
  end
  
  def test_event_handler_subscribes
    handles :test_event
    assert ScCore::MessageBus.handlers.size == 1
    assert ScCore::MessageBus.handlers[:test_event].size == 1
    assert_equal :test_event, ScCore::MessageBus.handlers[:test_event].first[:method]
  end
  
  def test_event_handler_handles_with_opts
    handles :test_event, :with => :test_event1
    assert ScCore::MessageBus.handlers.size == 1
    assert ScCore::MessageBus.handlers[:test_event].size == 1
    assert_equal :test_event1, ScCore::MessageBus.handlers[:test_event].first[:method]
  end
end
