require 'active_record'

module ScLock
  
  class LockNotAvailable < StandardError
  end
  
  class Lock < ActiveRecord::Base
    attr_accessible :resource_name, :key, :lock_expiry_time, :caller_key, :locked
    
    def lock(time_to_live)
      self.locked = true
      self.lock_expiry_time = Time.now.advance(:seconds => time_to_live)
    end
    
    def release
      self.locked = false
    end
  end
  
  def self.set_lock_db_config(config)
    Lock.establish_connection config
  end
  
  def lock(namespace, resource_name, params={}, caller_key=nil, time_to_live=600, wait_time=1000, retry_count=0, &block)
    resource_name = "#{namespace}-#{resource_name}"  if namespace && resource_name
    acquired = false
    first_run = true
    
    begin
      if !first_run
        sleep (wait_time/1000.0)
      end
      begin 
        caller_key = try_lock(resource_name, params, caller_key, time_to_live, &block)
        acquired = true
      rescue Exception => e
        #TODO Log & Suppress
      end
      retry_count -= 1
      first_run = false
    end while (!acquired && retry_count > 0)
    
    if acquired == false
      raise LockNotAvailable, "Lock not available"
    end
    
    if block
      error = nil
      begin 
        yield(caller_key)
      rescue Exception => e
        error = e
      end
      Lock.transaction(:requires_new => true) do
        lock = Lock.find_by_caller_key caller_key
        lock.locked = false
        lock.save!
      end
      raise error if error
    else
      return caller_key
    end
  end
  
  def unlock(caller_key)
    lock = Lock.find_by_caller_key(caller_key)
    lock.release
    remove_caller_key lock.resource_name, lock.key
    Lock.transaction(:requires_new => true) do
      lock.save!
    end
  end
  
  private
  def get_key(params)
    params.sort.to_s
  end
  
  def try_lock(resource_name, params, caller_key, time_to_live, &block)
    lock = nil
    key = get_key(params) if params
    reentrant = false
    if caller_key
      lock = Lock.find_by_caller_key(caller_key)
      reentrant = true if lock
    else
      lock = Lock.find_by_resource_name_and_key(resource_name, key)
    end
    
    if !caller_key
      caller_key = get_caller_key(resource_name, key) if !caller_key
      if caller_key
        reentrant = true
      end
    end
    caller_key = UUIDTools::UUID.random_create.to_s if !reentrant
        
    lock = Lock.new({:resource_name => resource_name, :key => key, :caller_key => caller_key}) if !lock

    if reentrant == false && lock.locked == true && lock.lock_expiry_time > Time.now
        raise LockNotAvailable, "Lock not available"
    end
    
    begin
      Lock.transaction(:requires_new => true) do
        if lock.id
          # Reentrant locks will return update count as 0
          count = Lock.update_all("caller_key = '#{caller_key}', locked=1, lock_expiry_time = now() + interval #{time_to_live} second", "id = #{lock.id} and (locked = 0 or lock_expiry_time < now())")
          if count <= 0 && !reentrant
            raise LockNotAvailable, "Lock not available"
          end
        else
          lock.lock(time_to_live)
          lock.save!
        end
      end
    rescue Exception => e
      # TODO log exception
      raise LockNotAvailable, e.inspect 
    end
    
    add_caller_key(lock.resource_name, lock.key, caller_key)
    return caller_key
  end
  
  def add_caller_key(resource_name, key, caller_key)
    Thread.current[:locks] = {} if !Thread.current[:locks]
    Thread.current[:locks][{resource_name => resource_name, :key => key}] = caller_key
  end
  
  def get_caller_key(resource_name, key)
    Thread.current[:locks][{resource_name => resource_name, :key => key}] if Thread.current[:locks] 
  end
  
  def remove_caller_key(resource_name, key)
    Thread.current[:locks].delete({resource_name => resource_name, :key => key}) if Thread.current[:locks]
  end
  
end
