require File.expand_path(File.dirname(__FILE__) + '/../constants/warehouse_constants.rb')
require File.expand_path(File.dirname(__FILE__) + '/../utils/logger.rb')
#Mixin of WarehouseOperations requires
#   : warehouse_client function implementation in the class which is mixin WarehouseOperations
#   : shelves function to return hash of shelf hashies

module SC
  module Validations
    module WarehouseValidations
      include SC::Constants::WarehouseConstants
      include SC::Constants::HttpStatusCodes
      include SC::TestCore::Logger
      FORWARD_AREA = "FORWARD_AREA"
      STORE = "STORE"

      #expected_response
      #[
      #    {
      #        :del => 0,
      #        :blr => 0,
      #        :fsn => "fsn",
      #        :sku => "sku"
      #    }
      #]
      def validate_successful_get_atp_response(get_atp_response, expected_response)
        assert_equal(STATUS_GET, get_atp_response.code, "Status Code")

        expected_response = {"inventories"=> expected_response}

        assert_equal(expected_response, JSON.parse(get_atp_response.body))
      end

      #get_and_expected_atp_entries
      #[
      #    {
      #        :fsn => @product1.fsn,
      #        :sku => @product1.sku,
      #        @del_warehouse.id => 0,
      #        @blr_warehouse.id => 2
      #    }
      #]
      #Send the list of expected atp entries. It does get and validate
      def validate_atp(get_and_expected_atp_entries)
        get_atp_request = get_and_expected_atp_entries.collect do |entry|
          {:fsn => entry[:fsn], :sku => entry[:sku]}
        end
        get_atp_response = get_atp(get_atp_request)

        validate_successful_get_atp_response(get_atp_response,
                                             JSON.parse(get_and_expected_atp_entries.to_json))
      end

      #expected_stock  {"inward_bulk" => 0, "st-1001" => 2}
      def validate_inventory_stock_take(warehouse_id, product, expected_stock)
        get_inventory_response  = warehouse_client.get_inventory(warehouse_id, product.fsn, product.sku)
        assert_equal(STATUS_GET,get_inventory_response.code,"Status Code")

        get_inventory_resp_json = JSON.parse(get_inventory_response.body)
        expected_stock.each do |location, stock|
          assert_true((get_inventory_resp_json.include?(location) and get_inventory_resp_json[location]==stock),
                      "#{location}=>#{stock} not found in observed stock inventory #{get_inventory_resp_json} for product #{product}")
        end
      end

      def validate_pending_reservation(warehouse_id, order_item_id, pending_reservation_id, product, expected_quantity, expected_status)
        expected_pending_reservation = {
            :id => pending_reservation_id,
            :fsn => product.fsn,
            :sku => product.sku,
            :order_item_id => order_item_id,
            :quantity => expected_quantity,
            :status => expected_status,
            :warehouse_id => warehouse_id
        }
        get_pen_res_resp = warehouse_client.get_pending_reservation(pending_reservation_id, warehouse_id)
        response = JSON.parse(get_pen_res_resp.body)["pending_reservations"]
        expected_pending_reservation["ship_by_date"] = response["ship_by_date"]
        assert_equal(JSON(expected_pending_reservation.to_json), JSON.parse(get_pen_res_resp.body)["pending_reservations"], "Pending Reservation")
      end

      #Can be used if you have context of create_pending reservation response
      def validate_pending_reservation_change(warehouse_id, create_pen_res_resp, expected_pending_quantity)
        create_pending_resp_json = JSON.parse(create_pen_res_resp.body)["pending_reservations"][0]
        create_pending_resp_json["quantity"] = expected_pending_quantity

        pending_res_id = create_pending_resp_json["id"]
        get_pen_res_resp = warehouse_client.get_pending_reservation(pending_res_id, warehouse_id)
        assert_equal(create_pending_resp_json, JSON.parse(get_pen_res_resp.body)["pending_reservations"], "Pending Reservation")
      end

      #expected_pickables
      #{
      #    :BOOK => 2,
      #    :AVMEDIA => 2
      #}
      def validate_expected_pickables(warehouse_id, location_type, expected_pickables, processing_method = "#{location_type}_picklist")
        get_pickables_response  = get_pickables!(warehouse_id, location_type, processing_method)
        get_pickables_resp_json  = JSON(get_pickables_response.body)
        logger.info  "Expected Pickables #{expected_pickables}"

        get_pickables_resp_json["ship_groups"].each do |ship_group|
          logger.info  "Remaining Ship Group #{ship_group}"
        end

        assert_equal(JSON(expected_pickables.to_json), get_pickables_resp_json, "Expected Pickables")
      end

      def validate_no_picklist_got_created(create_picklist_resp)
        #Validate Create Picklist Response
        assert_equal(STATUS_GET,create_picklist_resp.code,"Status Code")

        picklists_json = JSON.parse(create_picklist_resp.body)["picklists"]
        assert_equal(0,picklists_json.length,"Picklists")
      end

      #validate the present of picklist_item id and count. Max 8 pick items in every pick list
      def validate_picklist_item_id_present_and_unique(create_picklist_resp)
        observed_picklists_json = JSON.parse(create_picklist_resp.body)["picklists"].clone
        picklist_item_ids = []
        observed_picklists_json.each do |picklist_json|
          picklist_item_count = 0
          picklist_json["picklist_items"].each do |picklist_item|
            picklist_item_id = picklist_item["id"] # Use ful when i am going to check expected pick items check
            assert_kind_of(Integer,picklist_item_id,"Picklist Item id")
            assert_false(picklist_item_ids.include?(picklist_item_id),"picklist item id #{picklist_item_id} is duplicate")
            assert_equal(picklist_json["id"], picklist_item["picklist_id"],"picklist id at picklist and picklist item level")
            picklist_item_ids << picklist_item_id
            picklist_item_count +=  1
          end

          assert_true(picklist_item_count <= MAX_PICKLIST_SIZE, "Max Picklist items per picklist can be #{MAX_PICKLIST_SIZE} but they are #{picklist_item_count}")
        end
      end

      #Validate only picklist level meta details
      def validate_picklist_details(create_picklist_resp, expected_picklist_details)
        observed_picklists_json = JSON.parse(create_picklist_resp.body)["picklists"].clone

        observed_picklists_json.each do |picklist_json|
          assert_kind_of(Integer,picklist_json["id"],"Picklist Id")
          assert_equal(expected_picklist_details["picklist_type"],picklist_json["picklist_type"])
          assert_equal(expected_picklist_details["picklist_status"],picklist_json["picklist_status"])
          assert_equal(expected_picklist_details["warehouse_id"],picklist_json["warehouse_id"])
        end
      end

      #Validate expected picklist items details in create picklist response
      def validate_picklist_items_details(create_picklist_resp, expected_picklist_items, expected_picklist_item_count)
        validate_picklist_item_id_present_and_unique(create_picklist_resp)

        original_expected_picklist_items  = expected_picklist_items.clone

        #enhance expected picklist items
        original_expected_picklist_items.each do |expected_picklist_item|
          expected_picklist_item["picklist_item_status"] = "created"
          expected_picklist_item["quantity_picked"] = 0
          expected_picklist_item["quantity_to_pick"] = expected_picklist_item["quantity"]
          expected_picklist_item["product_title"] = @products[expected_picklist_item["fsn"]].product_title
          expected_picklist_item["has_serial_numbers"] = false
          expected_picklist_item.delete("quantity")
        end

        logger.info  "Original Expected Picklist Items #{original_expected_picklist_items}"

        observed_picklist_item_count = 0
        JSON.parse(create_picklist_resp.body)["picklists"].each do |picklist_content|

          picklist_content["picklist_items"].each do |observed_picklist_item|
            observed_picklist_item.delete("id")
            observed_picklist_item.delete("picklist_id")

            assert_true(original_expected_picklist_items.include?(observed_picklist_item),
                        "Observed Picklist item '#{observed_picklist_item}' not found in expected picklist items #{original_expected_picklist_items} picklist items")
            #deleting so that no duplicate check passes
            original_expected_picklist_items.delete_at(original_expected_picklist_items.index(observed_picklist_item))
            logger.info  "Renaiming Picklist Items #{original_expected_picklist_items.length} #{original_expected_picklist_items}"
            observed_picklist_item_count += 1
          end
        end

        assert_equal(expected_picklist_item_count, observed_picklist_item_count, "Expected Picklist items in all created picklists")
      end

      #Check All Picklist ids and correspoding picklist_items
      def validate_picklist_creation_response(create_picklist_resp, expected_picklist_count, expected_picklist_items,
          expected_picklist_details, expected_picklist_item_count)

        assert_equal(STATUS_CREATED,create_picklist_resp.code,"Status Code")

        observed_picklists_json = JSON.parse(create_picklist_resp.body)["picklists"].clone

        assert_equal(expected_picklist_count,observed_picklists_json.length,"Created Picklists")

        validate_picklist_details(create_picklist_resp, expected_picklist_details)

        validate_picklist_items_details(create_picklist_resp, expected_picklist_items, expected_picklist_item_count)
      end

      def validate_successful_cancel_order_item_response(cancel_order_item_resp, expected_cancellations)
        assert_equal(STATUS_DELETED, cancel_order_item_resp.code, "Cancel Order Item Response Code")
        assert_equal(JSON({:cancelled_items => expected_cancellations}.to_json), JSON(cancel_order_item_resp.body))
      end

      def validate_successful_warehouse_confirm_shipment(confirm_shipment_resp)
        assert_equal(STATUS_UPDATED,confirm_shipment_resp.code,"Status Code")
      end

      def validate_successful_reprocess_shipment_resp(reprocess_shipment_resp, cancelled_items, ready_to_ship_items)
        reprocess_shipment_resp_json = JSON(reprocess_shipment_resp.body)
        new_shipment_id = reprocess_shipment_resp_json.delete("new_shipment_id")
        assert_kind_of(Integer,new_shipment_id,"New Shipment Id #{new_shipment_id} in reprocess response")

        expected_reponse = {
            :actions => {
                :cancelled_items => cancelled_items.collect {|cancelled_item| {:title => cancelled_item[:product].product_title, :quantity => cancelled_item[:quantity], :fsn => cancelled_item[:product].fsn}},
                :ready_to_ship_items => ready_to_ship_items.collect {|ready_to_ship_item| {:title => ready_to_ship_item[:product].product_title, :quantity => ready_to_ship_item[:quantity], :fsn => ready_to_ship_item[:product].fsn}}
            }
        }
        assert_equal(JSON(expected_reponse.to_json),reprocess_shipment_resp_json,"Reprocess Response")
        new_shipment_id
      end

      def validate_successful_reprocess_shipment_resp_without_new_shipment_id(reprocess_shipment_resp, cancelled_items)
        reprocess_shipment_resp_json = JSON(reprocess_shipment_resp.body)
        expected_reponse = {
            :actions => {
                :cancelled_items => cancelled_items.collect {|cancelled_item| {:title => cancelled_item[:product].product_title, :quantity => cancelled_item[:quantity], :fsn => cancelled_item[:product].fsn}},
                :ready_to_ship_items => []
            }
        }
        assert_equal(JSON(expected_reponse.to_json),reprocess_shipment_resp_json,"Reprocess Response")
      end

      def validate_confirm_shipment_response_in_order_cancellation_case(shipment_id, warehouse_confirm_shipment_resp)
        assert_equal(STATUS_BAD_REQUEST, warehouse_confirm_shipment_resp.code, "Confirm Shipment response when order has been already cancelled")
        validate_error_message(warehouse_confirm_shipment_resp.body,
                               "ERROR",
                               "SHIPMENT_CANCELLED",
                               {"shipment_id" => shipment_id},
                               "Shipment ID - #{shipment_id} has been cancelled. Move it to Pack-Ship Transit.")
      end

      def validate_pending_reservation_existence(order_item_id, product, quantity, status = "created")
        pen_res_resp_json= get_pending_reservations_by_order_id!(order_item_id)[1]
        pen_res_resp_json["pending_reservations"].each do |pending_reservation|
          return if pending_reservation["order_item_id"] = order_item_id and
              pending_reservation["quantity"] = quantity and
              pending_reservation["fsn"] = product.fsn and
              pending_reservation["sku"] == product.sku and
              pending_reservation["status"] == status
        end
        assert_true(false, "No pending reservation found with order_item_id #{order_item_id} fsn #{product.fsn} sku #{product.sku} status = #{status}and quantity #{quantity}")
      end

    end
  end
end
