require_relative "base_model_test"

class ApCacheTest < BaseModelTest

  context 'Test check_api_disabled method' do

    context 'Empty API Cache' do
      setup do
        class Foo
          extend ApiCache
        end
      end

      should 'have an empty cache' do
        assert_equal [], Foo.api_config
      end

      should 'return false for any BU, any method any uri' do
        result = Foo.check_api_disabled('WSR', 'POST', '/invoice/type/revenue_invoice')
        assert_equal false, result

        result = Foo.check_api_disabled('EKL', 'DELETE', '/invoice/type/revenue_invoice/INV1234')
        assert_equal false, result

        result = Foo.check_api_disabled('FKMP', 'PUT', '/invoice/type/cost_invoice/INV1234/documents/DOC123')
        assert_equal false, result

        result = Foo.check_api_disabled('FKI', 'GET', '/invoice/type/cost_invoice/INV1234')
        assert_equal false, result
      end
    end

    context 'Non-Empty API Cache - Block entire BU' do
      setup do
        class Foo
          extend ApiCache
        end
        Foo.api_config=([{'service_name' => 'apl_invoice_register', 'bu' => 'wsr'}])

      end

      should 'returns true for any api in WSR' do
        result = Foo.check_api_disabled('wsr', 'POST', '/type/revenue_invoice/')
        assert_equal true, result

        result = Foo.check_api_disabled('wsr', 'GET', '/type/revenue_invoice/123')
        assert_equal true, result

        result = Foo.check_api_disabled('wsr', 'PUT', '/type/revenue_invoice/123')
        assert_equal true, result

        result = Foo.check_api_disabled('wsr', 'DELETE', '/type/revenue_invoice/123')
        assert_equal true, result
      end

      should 'return false for other BUs same apis' do
        result = Foo.check_api_disabled('ekl', 'POST', '/type/revenue_invoice/')
        assert_equal false, result

        result = Foo.check_api_disabled('ekl', 'GET', '/type/revenue_invoice/123')
        assert_equal false, result

        result = Foo.check_api_disabled('ekl', 'PUT', '/type/revenue_invoice/123')
        assert_equal false, result

        result = Foo.check_api_disabled('ekl', 'DELETE', '/type/revenue_invoice/123')
        assert_equal false, result
      end

    end

    context 'Non-Empty API Cache - Block specific method type in a BU.' do
      setup do
        class Foo
          extend ApiCache
        end
        # All the values have to be in lower case
        Foo.api_config=([{'service_name' => 'apl_invoice_register', 'bu' => 'wsr', 'method' => 'post'}])
      end

      should 'return true for all POST calls for WSR BU' do
        result = Foo.check_api_disabled('wsr', 'POST', '/type/revenue_invoice/')
        assert_equal true, result

        result = Foo.check_api_disabled('wsr', 'POST', '/documents')
        assert_equal true, result

        result = Foo.check_api_disabled('wsr', 'POST', '/type/foo_invoice')
        assert_equal true, result
      end

      should 'return false for other method types in WSR BU' do
        result = Foo.check_api_disabled('wsr', 'GET', '/type/revenue_invoice/')
        assert_equal false, result

        result = Foo.check_api_disabled('wsr', 'PUT', '/type/cost_invoice')
        assert_equal false, result

        result = Foo.check_api_disabled('wsr', 'DELETE', '/type/foo_invoice')
        assert_equal false, result
      end

      should 'return false for POST and other method types for other BUs' do
        result = Foo.check_api_disabled('ekl', 'POST', '/type/revenue_invoice/')
        assert_equal false, result

        result = Foo.check_api_disabled('ekl', 'GET', '/type/revenue_invoice/')
        assert_equal false, result

        result = Foo.check_api_disabled('ekl', 'PUT', '/type/cost_invoice')
        assert_equal false, result

        result = Foo.check_api_disabled('ekl', 'DELETE', '/type/foo_invoice')
        assert_equal false, result
      end
    end

    context 'Non-Empty API Cache - Block specific method type across all BU.' do
      setup do
        class Foo
          extend ApiCache
        end
        # All the values have to be in lower case
        Foo.api_config=([{'service_name' => 'apl_invoice_register', 'method' => 'post' }])
      end

      should 'return true for post call for /type/foo_invoice across all BUs' do
        result = Foo.check_api_disabled('WSR', 'POST', '/type/foo_invoice')
        assert_equal true, result

        result = Foo.check_api_disabled('EKL', 'POST', '/documents')
        assert_equal true, result

        result = Foo.check_api_disabled('FKI', 'POST', '/generate_data')
        assert_equal true, result

        result = Foo.check_api_disabled('FKMP', 'POST', '/kill_monster')
        assert_equal true, result
      end

      should 'return false for other calls' do
        result = Foo.check_api_disabled('EKL', 'PUT', '/type/dummy_invoice/D1234')
        assert_equal false, result

        result = Foo.check_api_disabled('FKI', 'DELETE', '/type/dummy_invoice/D1234/document/DF123')
        assert_equal false, result

        result = Foo.check_api_disabled('FKMP', 'GET', '/type/dummy_invoice/D1234/items/DF123')
        assert_equal false, result
      end
    end

    context 'Non-Empty API Cache - Block specific method type and uri in a BU.' do
      setup do
        class Foo
          extend ApiCache
        end
        # All the values have to be in lower case
        Foo.api_config=([{'service_name' => 'apl_invoice_register',
                          'bu' => 'wsr', 'method' => 'post', 'uri' => '/type/foo_invoice' }])
      end

      should 'return true for wsr post call for /type/foo_invoice' do
        result = Foo.check_api_disabled('WSR', 'POST', '/type/foo_invoice')
        assert_equal true, result
      end

      should 'return false for other bu post call for /type/foo_invoice' do
        result = Foo.check_api_disabled('EKL', 'POST', '/type/foo_invoice')
        assert_equal false, result
      end
    end


    context 'Non-Empty API Cache - Block specific method type and uri across all BUs.' do
      setup do
        class Foo
          extend ApiCache
        end
        # All the values have to be in lower case
        Foo.api_config=([{'service_name' => 'apl_invoice_register', 'method' => 'post',
                          'uri' => '/type/foo_invoice' }])
      end

      should 'return true for all BUs' do
        result = Foo.check_api_disabled('EKL', 'POST', '/type/foo_invoice')
        assert_equal true, result

        result = Foo.check_api_disabled('WSR', 'POST', '/type/foo_invoice')
        assert_equal true, result

        result = Foo.check_api_disabled('FKI', 'POST', '/type/foo_invoice')
        assert_equal true, result

        result = Foo.check_api_disabled('FKMP', 'POST', '/type/foo_invoice')
        assert_equal true, result
      end

      should 'return false for any other method/uri/bu ' do
        result = Foo.check_api_disabled('EKL', 'PUT', '/type/foo_invoice/GTR123/documents/DC123')
        assert_equal false, result

        result = Foo.check_api_disabled('WSR', 'GET', '/type/foo_invoice/DVB1234')
        assert_equal false, result

        result = Foo.check_api_disabled('FKI', 'POST', '/type/revenue_invoice')
        assert_equal false, result

        result = Foo.check_api_disabled('FKMP', 'DELETE', '/type/revenue_invoice/REV1234')
        assert_equal false, result
      end

    end
  end
end


