class Lock < ActiveRecord::Base
end

class ResourceManagerTest < BaseModelTest
  context "lock not available" do
    setup do
      @resource = "test-resource"
      @params = {'key1' => 'value1', 'key2' => 'value2'}
      ResourceManager.lock_resource(@resource, @params) {}
    end

    should "get proper exception error on attempting to take same lock" do
      #making lock non-rentrant
      Thread.current[:locks] = nil
      lock = Lock.first
      lock.locked = true
      lock.save!
      assert_raise ResourceLockedError do
        ResourceManager.lock_resource(@resource, @params) {}
      end
    end

    should "get proper 423 error message (for restbus retry)" do
      #making lock non-rentrant
      begin
        Thread.current[:locks] = nil
        lock = Lock.first
        lock.locked = true
        lock.save!
        ResourceManager.lock_resource(@resource, @params) {}
      rescue ResourceLockedError => e
        assert_equal 423, e.http_error_code
      end

    end
  end

  context 'Locker' do
    context 'take lock on all given entities' do
      setup do
        entities = [{:client_ref_id => 12}, {:client_ref_id => 23}]
        @locked_entities = ResourceManager.lock_for_each 'test-service', 'entities', entities, :client_ref_id,  60*60
      end

      should 'be able to get lock on all entities' do
        assert_equal 2, @locked_entities.count
        an_hour_from_now = (Time.now + (60*60)).to_s
        assert_equal an_hour_from_now, ScLock::Lock.all.first.lock_expiry_time.to_s
      end
    end

    context 'Similar Resource lock' do

      should 'allow only one to take mutex' do
        entities = [{:id => 12}, {:id => 23}]
        threads = []
        20.times do
          threads << Thread.new { ResourceManager.lock_for_each 'test-service', 'entities', entities }
        end
        threads.each do |thread|
          thread.join
        end
        assert_equal 2, ScLock::Lock.all.count
      end
    end

    context 'Update the expiry date for locks ' do
      should 'update the expiry date' do
        entities = [{:id => 12}, {:id => 23}]
        begin
          ResourceManager.lock_for_each 'test-service', 'entities', entities
        rescue ScLock::LockNotAvailable => e
        end
        assert_equal 2, ScLock::Lock.all.count
        assert_equal Date.tomorrow.to_s, Date.parse(ScLock::Lock.all.first.lock_expiry_time.to_s).to_s
        a_month = Date.current + 30.days
        ResourceManager.update_expiry_date_for_all 'test-service', 'entities', entities, a_month
        assert_equal (Date.current + 30.days).to_s, Date.parse(ScLock::Lock.all.first.lock_expiry_time.to_s).to_s
      end
    end


    context 'Test Distributed lock' do
      def dummy_entries
        @count = 0
        ResourceManager.lock('test-lock', '123') do
          if @count == 0
            @count += 1
          end
        end
      end

      should 'create only one dummy entry' do
        threads = []
        5.times do
          threads << Thread.new { dummy_entries }
        end
        threads.each do |thread|
          begin
            thread.join
          rescue ScLock::LockNotAvailable => e
            next
          end
        end
        assert_equal 1, @count
      end
    end
  end
end