# encoding: utf-8

require "spec_helper"

describe Money do

  describe ".new" do
    it "accepts optional currency" do
      m = Money.new(1)
      m.currency.should == Money.default_currency
    end

    it "rejects arbitrary objects as amounts" do
      lambda { Money.new([]) }.should raise_error(StandardError, /Invalid amount/)
      lambda { Money.new(nil) }.should raise_error(StandardError, /Invalid amount/)
      lambda { Money.new("i") }.should raise_error(StandardError, /Invalid amount/)
    end

    it "accepts optional bank" do
      m = Money.new(1)
      m.bank.should == Money.default_bank

      m = Money.new(1, "EUR", bank = Object.new)
      m.bank.should == bank
    end

    it "rounds to number of decimal places" do
      Money.new("1.014").amount.should == "1.01".to_d
      Money.new("1.015").amount.should == "1.02".to_d
    end

    it "rounds using ROUND_HALF_UP " do
      old_mode = BigDecimal.mode(BigDecimal::ROUND_MODE)
      begin
        BigDecimal.mode(BigDecimal::ROUND_MODE, BigDecimal::ROUND_HALF_EVEN)
        Money.new("1.025").amount.should == "1.03".to_d
      ensure
        BigDecimal.mode(BigDecimal::ROUND_MODE, old_mode)
      end
    end

    it "rounds to currency-specific decimal places" do
      Money.new("1.0015", "KWD").amount.should == "1.002".to_d
      Money.new("1.015", "INR").amount.should == "1.02".to_d
      Money.new("1.15", "VND").amount.should == "1.2".to_d
      Money.new("1.5", "JPY").amount.should == "2".to_d
    end

    it "accepts Numerics" do
      Money.new(1.015).amount.should be_decimal("1.02")
      Money.new(1).amount.should == "1".to_d
      Money.new(BigDecimal.new("1.125")).amount.should == "1.13".to_d
    end

    it "rejects anything that isn't a string or Numeric" do
      lambda { Money.new("not a number") }.should raise_exception(StandardError, /Invalid amount/)
    end

    it "is associated to the singleton instance of Bank::VariableExchange by default" do
      Money.new(0).bank.should be(Money::Bank::VariableExchange.instance)
    end
  end

  describe ".empty" do
    it "creates a new Money object of 0 amount" do
      Money.empty.should == Money.new(0)
      Money.zero.should == Money.new(0)
    end
  end

  describe ".ca_dollar" do
    it "creates a new Money object of the given value in CAD" do
      Money.ca_dollar(50).should == Money.new(50, "CAD")
    end
  end

  describe ".inr" do
    it "creates a new Money object of the given value in INR" do
      Money.inr(50).should == Money.new(50, "INR")
    end
  end

  describe ".us_dollar" do
    it "creates a new Money object of the given value in USD" do
      Money.us_dollar(50).should == Money.new(50, "USD")
    end
  end

  describe ".euro" do
    it "creates a new Money object of the given value in EUR" do
      Money.euro(50).should == Money.new(50, "EUR")
    end
  end

  describe ".add_rate" do
    it "saves rate into current bank" do
      Money.add_rate("EUR", "USD", 2)
      Money.new(1000, "EUR").exchange_to("USD").should == Money.new(2000, "USD")
    end
  end

  describe "#currency" do
    it "returns the currency object" do
      Money.new(1_00, "USD").currency.should == Money::Currency.new("USD")
    end
  end

  describe "#currency_as_string" do
    it "returns the iso_code of the currency object" do
      Money.new(1_00, "USD").currency_as_string.should == "USD"
      Money.new(1_00, "EUR").currency_as_string.should == "EUR"
    end
  end

  describe "#currency_as_string=" do
    it "sets the currency object using the provided string" do
      money = Money.new(100_00, "USD")
      money.currency_as_string = "EUR"
      money.currency.should == Money::Currency.new("EUR")
      money.currency_as_string = "YEN"
      money.currency.should == Money::Currency.new("YEN")
    end
  end

  describe "#hash=" do
    it "returns the same value for equal objects" do
      Money.new(1.00, "EUR").hash.should == Money.new(1.00, "EUR").hash
      Money.new(2.00, "USD").hash.should == Money.new("2.00", "USD").hash
      Money.new(1.00, "EUR").hash.should_not == Money.new(2.00, "EUR").hash
      Money.new(1.00, "EUR").hash.should_not == Money.new(1.00, "USD").hash
      Money.new(1.00, "EUR").hash.should_not == Money.new("2.00", "USD").hash
    end

    it "can be used to return the intersection of Money object arrays" do
      intersection = [Money.new(100, "EUR"), Money.new(100, "USD")] & [Money.new(100, "EUR")]
      intersection.should == [Money.new(100, "EUR")]
    end
  end

  describe "#symbol" do
    it "works as documented" do
      currency = Money::Currency.new("EUR")
      currency.should_receive(:symbol).and_return("€")
      Money.empty(currency).symbol.should == "€"

      currency = Money::Currency.new("EUR")
      currency.should_receive(:symbol).and_return(nil)
      Money.empty(currency).symbol.should == "¤"
    end
  end

  describe "#to_s" do
    it "works as documented" do
      Money.new("10.00").to_s.should == "10.00"
      Money.new("400.08").to_s.should == "400.08"
      Money.new(400.08).to_s.should == "400.08"
      Money.new(-237.43).to_s.should == "-237.43"
      Money.new(10, "JPY").to_s.should == "10"
    end

    it "respects :decimal_mark" do
      Money.new("10.00", "BRL").to_s.should == "10,00"
      Money.new("1000.00", "BRL").to_s.should == "1000,00"
    end
  end

  describe "#to_d" do
    it "works as documented" do
      Money.new(10.00).to_d.should be_decimal("10.0")
    end
  end

  describe "#to_f" do
    it "works as documented" do
      Money.new(10.00).to_f.should == 10.0
    end
  end

  describe "#to_money" do
    it "works as documented" do
      money = Money.new(10.00, "DKK")
      money.should == money.to_money
      money.should == money.to_money("DKK")
      money.bank.should_receive(:exchange_with).with(Money.new(10.00, Money::Currency.new("DKK")), Money::Currency.new("EUR")).and_return(Money.new(200.00, Money::Currency.new('EUR')))
      money.to_money("EUR").should == Money.new(200.00, "EUR")
    end
  end

  describe "#exchange_to" do
    it "exchanges the amount via its exchange bank" do
      money = Money.new(100_00, "USD")
      money.bank.should_receive(:exchange_with).with(Money.new(100_00, Money::Currency.new("USD")), Money::Currency.new("EUR")).and_return(Money.new(200_00, Money::Currency.new('EUR')))
      money.exchange_to("EUR")
    end

    it "exchanges the amount properly" do
      money = Money.new(100_00, "USD")
      money.bank.should_receive(:exchange_with).with(Money.new(100_00, Money::Currency.new("USD")), Money::Currency.new("EUR")).and_return(Money.new(200_00, Money::Currency.new('EUR')))
      money.exchange_to("EUR").should == Money.new(200_00, "EUR")
    end
  end

  describe "#allocate" do
    it "takes no action when one gets all" do
      Money.us_dollar(5).allocate([1.0]).should == [Money.us_dollar(5)]
    end

    it "keeps currencies intact" do
      Money.ca_dollar(5).allocate([1]).should == [Money.ca_dollar(5)]
    end

    it "does not loose pennies" do
      moneys = Money.us_dollar(0.05).allocate([0.3, 0.7])
      moneys[0].should == Money.us_dollar(0.02)
      moneys[1].should == Money.us_dollar(0.03)
    end

    it "does not loose pennies too" do
      moneys = Money.us_dollar(1.00).allocate([0.33, 0.33, 0.33])
      moneys[0].amount.should == 0.34
      moneys[1].amount.should == 0.33
      moneys[2].amount.should == 0.33
    end

    it "requires total to be less then 1" do
      expect { Money.us_dollar(0.05).allocate([0.5, 0.6]) }.to raise_error(ArgumentError)
    end
  end

  describe "#split" do
    it "needs at least one party" do
      expect { Money.us_dollar(0.01).split(0) }.to raise_error(ArgumentError)
      expect { Money.us_dollar(0.01).split(-1) }.to raise_error(ArgumentError)
    end

    it "gives 1 cent to both people if we start with 2" do
      Money.us_dollar(0.02).split(2).should == [Money.us_dollar(0.01), Money.us_dollar(0.01)]
    end

    it "may distribute no money to some parties if there isn't enough to go around" do
      Money.us_dollar(0.002).split(3).should == [Money.us_dollar(0.001), Money.us_dollar(0.001), Money.us_dollar(0)]
    end

    it "does not lose pennies" do
      Money.us_dollar(0.1).split(3).should == [Money.us_dollar(0.04), Money.us_dollar(0.03), Money.us_dollar(0.03)]
      Money.us_dollar(0.05).split(2).should == [Money.us_dollar(0.03), Money.us_dollar(0.02)]
    end

    it "splits a dollar" do
      moneys = Money.us_dollar(1.00).split(3)
      moneys[0].amount.should == 0.34
      moneys[1].amount.should == 0.33
      moneys[2].amount.should == 0.33
    end
  end

end
