require 'sc-rest-client/rest_helpers'

require 'forwardable'

module Padrino
  # Delegates to Padrino.logger, which maintains a thread-local logger object. Without this, when logger object references
  # are assigned and cached (like ActiveRecord::Base.log = Padrino.logger), global changes to log-levels won't
  # reflect in the cached logger instance.
  #
  # NOTE: The set of delegated methods must be updated when upgrading Padrino
  #
  class DelegatingLogger
    extend Forwardable
    def_delegators :'Padrino.actual_logger', :level, :level=, :auto_flush, :auto_flush=, :log_static, :log_static=
    def_delegators :'Padrino.actual_logger', :buffer, :log, :init_args
    def_delegators :'Padrino.actual_logger', :colored_level, :flush, :close, :push, :bench, :<<, :write
    def_delegators :'Padrino.actual_logger', :devel, :devel?, :debug, :debug?, :info, :info?, :warn, :warn?, :error, :error?, :fatal, :fatal?
  end

  class << self
    # replaces the logger method to return the delegator
    def logger
      @delegating_logger ||= DelegatingLogger.new
    end

    # this is the original logger method, with some changes to ensure that a new thread also uses the correct log-level
    def actual_logger
      Padrino::Logger.setup! if Thread.current[:padrino_logger].nil?

      # ensure logger's level is from Config (allows change to log-level at runtime - see AdminApp)
      Thread.current[:padrino_logger].level = Padrino::Logger::Levels[Padrino::Logger::Config[PADRINO_ENV.to_sym][:log_level]]

      Thread.current[:padrino_logger]
    end
  end

  class Logger
    include RestClient::RestHelpers

    ##
    # Extension to Padrino::Logger to print db-metrics, request body, response body, etc.
    #
    # Padrino::Loggger::Rack forwards every request to an +app+ given, and
    # logs a line in the Apache common log format to the +logger+, or
    # rack.errors by default.
    #
    class Rack

      def initialize(app, uri_root) # @private
        @app = app
        @uri_root = uri_root.sub(/\/$/, "")
      end

      def call(env) # @private
        env['rack.logger'] = Padrino.logger
        env['rack.errors'] = Padrino.logger.log
        began_at = Time.now
        status, header, body = @app.call(env)
        log(env, status, header, body, began_at)
        [status, header, body]
      end

      private
      def log(env, status, header, body, began_at)
        return if env['sinatra.static_file'] and !logger.log_static
        logger.bench(
            env["REQUEST_METHOD"],
            began_at,
            [
                @uri_root.to_s,
                env["PATH_INFO"],
                env["QUERY_STRING"].empty? ? "" : "?" + env["QUERY_STRING"],
                ' - ',
                status.to_s[0..3].bold,
                ' ',
                code_to_name(status)
            ] * '',
            :debug,
            :magenta
        )

        RequestResponseLogger.log(env, status, header, body) if ENV["ENABLE_REQ_RESP_LOGGING"]
      end

      def code_to_name(status)
        ::Rack::Utils::HTTP_STATUS_CODES[status.to_i] || ''
      end
    end # Rack

    def push(message = nil, level = nil)
      write @format_message % [colored_level(level), Time.now.strftime(@format_datetime).yellow, construct_message(message)]
    end

    def bench(action, began_at, message, level=:debug, color=:yellow)
      duration = Time.now - began_at
      color = :red if duration > 0.1
      if defined?(RackMetrics) && RackMetrics.current_request && RackMetrics.current_request.respond_to?(:total_query_time) # ensure rack-metrics version is correct
        db_time = RackMetrics.current_request.total_query_time
        db_queries = RackMetrics.current_request.total_query_count
        push "%s [" % action.to_s.upcase.send(color) +
                 "%0.3fs ".bold.send(color) % duration +
                 "DB: %0.3fs ".bold.send(color) % db_time +
                 "(%4.1f%%) ".bold.send(color) % (db_time / duration * 100.0) +
                 "%s queries".bold.send(color) %  + db_queries +
                 "] %s" % message.to_s,
             level
      else
        push("%s (" % action.to_s.upcase.send(color) + "%0.4fms".bold.send(color) % duration + ") %s" % message.to_s, level)
      end
    end

    private
    def construct_message(message)
      " [#{SupplyChain.service_name_with_company} #{current_transaction_id}#{current_request_id ? " #{current_request_id}" : ""}] - #{message.to_s}"
    end
  end
end

class RequestResponseLogger
  class << self
    def log(env, status, header, body)
      log_request_body(env)
      log_response_body(status, header, body)
    end

    private
    def pretty_generate(string)
      return JSON.pretty_generate(JSON.parse(string))
    rescue JSON::ParserError
      return string
    end

    def log_request_body(env)
      logger.push "Request: #{resource_url(env)} from #{caller_info(env)}", :debug
      env['rack.input'].rewind
      logger.push pretty_generate(env['rack.input'].read), :debug
    end

    def resource_url(env)
      base = "#{env['REQUEST_METHOD']} #{env['rack.url_scheme']}://#{env['SERVER_NAME']}:#{env['SERVER_PORT']}#{env['PATH_INFO']}"
      query_string = env['QUERY_STRING'] ? "" : "?" + env['QUERY_STRING']
      base + query_string
    end

    def caller_info(env)
      env['REMOTE_ADDR']
    end

    def log_response_body(status, header, body)
      logger.push "Response: #{status}, headers #{header}", :debug
      logger.push pretty_generate(body_to_string(body)), :debug
    end

    def log_multiline(string)
      string.each_line do |line|
        logger.push line, :debug
      end
    end

    def body_to_string(body)
      if body.is_a?(File)
        File.path(body)
      elsif body.is_a?(Array)
        body.join
      else
        body.to_s
      end
    end
  end
end
