$:.unshift File.expand_path(File.dirname(__FILE__) + '/../lib')

require 'test/unit'
require 'sc_core/supply_chain'
require 'sc-lock/lock'
require 'uuidtools'

module ScLockTest
  class Lock < ActiveRecord::Base
      
      establish_connection(
        :adapter   => RUBY_PLATFORM == 'java' ? 'mysql' : 'mysql2',
        :encoding  => 'utf8',
        :reconnect => true,
        :database  => 'oms_b2c_test',
        :pool      => 5,
        :username  => 'root',
        :password  => '',
        :host  => 'localhost'
      )
    
      attr_accessible :resource_name, :key, :lock_expiry_time, :caller_key, :locked
  end
end

ScLock.set_lock_db_config ({
        :adapter   => RUBY_PLATFORM == 'java' ? 'mysql' : 'mysql2',
        :encoding  => 'utf8',
        :reconnect => true,
        :database  => 'oms_b2c_test',
        :pool      => 5,
        :username  => 'root',
        :password  => '',
        :host  => 'localhost'
})

class LockTest < Test::Unit::TestCase
  
  include ScLock
  
  def setup
    Lock.delete_all
  end
  
  def test_lock
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
  end
  
  def test_lock_with_caller_key
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'}, "test1234")
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
  end
  
  def test_lock_not_available
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now

    Thread.current[:locks] = nil    
    assert_raise LockNotAvailable do 
      lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    end
  end
  
  def test_lock_not_available_with_caller_key
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'}, "test1234")
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
    
    Thread.current[:locks] = nil    
    assert_raise LockNotAvailable do 
      lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    end
  end
  
  def test_unlock
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
    
    unlock caller_key
    lock = Lock.find_by_caller_key caller_key
    assert !lock.locked
  end
  
  def test_renentrant_lock
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
    
    lock(nil, nil, nil, caller_key)
    
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
  end
  
  def test_renentrant_lock_without_caller_key
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
    
    lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
  end
  
  def test_lock_block
    key = nil
    lock "namespace", "test", {'key1'=>'value1', 'key2'=>'value2'} do |caller_key|
      assert_not_nil caller_key
      key = caller_key
    end
    lock = Lock.find_by_caller_key key
    assert !lock.locked
  end
  
  def test_lock_block_with_caller_key
    key = nil
    lock "namespace", "test", {'key1'=>'value1', 'key2'=>'value2'}, "abc" do |caller_key|
      assert_not_nil caller_key
      key = caller_key
    end
    lock = Lock.find_by_caller_key key
    assert !lock.locked
  end
  
  def test_lock_within_transaction_outer_throwing_exception
    lockt = ScLockTest::Lock.new({:resource_name => 'resource_name', :key => {}, :caller_key => 'caller_key', :lock_expiry_time => Time.now, :locked=>true})
    caller_key1 = nil
    assert_raise Exception do
      ScLockTest::Lock.transaction do
        assert lockt.save!
        caller_key1 = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
        assert_not_nil caller_key1
        lock1 = Lock.find_by_caller_key caller_key1
        assert lock1.locked
        assert lock1.lock_expiry_time > Time.now
        raise Exception
      end
    end
    lock = ScLockTest::Lock.find_by_caller_key 'caller_key'
    assert_nil lock
    lock1 = Lock.find_by_caller_key caller_key1
    assert_not_nil lock1
    assert lock1.locked
  end
  
  def test_lock_key_ordering
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
    
    Thread.current[:locks] = nil
    assert_raise LockNotAvailable do
      lock("namespace", "test", {'key2'=>'value2', 'key1'=>'value1'})
    end
  end
  
  def test_lock_with_retry
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
    
    Thread.current[:locks] = nil    
    assert_raise LockNotAvailable do
      lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'}, nil, 600, 1, 3)
    end
  end
  
  def test_lock_with_retry_and_wait_time
    caller_key = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'})
    assert_not_nil caller_key
    lock = Lock.find_by_caller_key caller_key
    assert lock.locked
    assert lock.lock_expiry_time > Time.now
    
    Thread.current[:locks] = nil    
    assert_raise LockNotAvailable do
      lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'}, nil, 600, 1, 3)
    end
  end

  def test_taking_a_lock_from_two_different_threads
    caller_key1 = lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'}) do end
    thread1_locks = Thread.current[:locks]
    Thread.current[:locks] = nil

    # Assume thread 2 is holding the lock with a different caller key
    caller_key2 = UUIDTools::UUID.random_create.to_s
    Lock.first.update_attributes!(:locked => 1, :caller_key => caller_key2)

    # Note that in thread1 Thread.current[:locks] is still set even after the
    # lock is released
    Thread.current[:locks] = thread1_locks

    # Thread 1 tries to get the lock and should fail
    assert_raise LockNotAvailable do
      lock("namespace", "test", {'key1'=>'value1', 'key2'=>'value2'}) do end
    end
  end
end
